package com.yao.common.aliy;

import com.aliyun.ecs20140526.Client;
import com.aliyun.ecs20140526.models.*;
import com.aliyun.tea.TeaException;
import com.aliyun.teautil.Common;
import com.aliyun.teautil.models.RuntimeOptions;
import com.yao.common.aliy.model.EcsSelectModel;
import com.yao.common.config.AlyConfigProperties;
import com.yao.common.domain.aliy.InstanceInfo;
import com.yao.gateway.cache.GatewayNodeSetVinCache;
import com.yao.common.redis.service.RedisService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * @Author: LiJiaYao
 * @Date: 2024/4/16
 * @Description: 阿里云ECS服务器openAPI调用
 */
@Component
@Log4j2
public class AliYunEcsService {

    private final AlyConfigProperties alyConfigProperties;
    private final Client client;

    private final GatewayNodeSetVinCache gatewayNodeSetVinCache;
    public AliYunEcsService(AlyConfigProperties alyConfigProperties, Client client, GatewayNodeSetVinCache gatewayNodeSetVinCache) {
        this.alyConfigProperties = alyConfigProperties;
        this.client = client;
        this.gatewayNodeSetVinCache = gatewayNodeSetVinCache;
    }

    //todo----------------------------------------------------以下是查询代码--------------------------------------------

    /**
     * @param
     * @return
     * @throws Exception
     * @Description: 根据id和name查询内容
     */
    public List<InstanceInfo> selectList(EcsSelectModel ecsSelectModel) throws Exception {
        DescribeInstancesRequest describeInstancesRequest = new DescribeInstancesRequest()
                .setRegionId(alyConfigProperties.getRegionId());
        if (ecsSelectModel.getInstanceNameList() == null || ecsSelectModel.getInstanceNameList().isEmpty()) {
            describeInstancesRequest.setInstanceName("*");
        } else {
            describeInstancesRequest.setInstanceName(Common.toJSONString(ecsSelectModel.getInstanceNameList()));
        }
        if (ecsSelectModel.getInstanceIdList() != null || !ecsSelectModel.getInstanceIdList().isEmpty()) {
            describeInstancesRequest.setInstanceIds(Common.toJSONString(ecsSelectModel.getInstanceIdList()).toString());
        } else {
            describeInstancesRequest.setInstanceName(Common.toJSONString(ecsSelectModel.getInstanceNameList()));
        }
        describeInstancesRequest.setPageSize(10);

        RuntimeOptions runtime = new RuntimeOptions();

        //初始化返回值
        List<DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstancesInstance> instanceList = null;

        try {
            // 复制代码运行请自行打印 API 的返回值
            DescribeInstancesResponse describeInstancesResponse = client.describeInstancesWithOptions(describeInstancesRequest, runtime);
            DescribeInstancesResponseBody body = describeInstancesResponse.getBody();
            DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstances instances = body.getInstances();
            //能返回的值
            instanceList = instances.getInstance();
            if (instanceList == null || instanceList.isEmpty()) {
                return new ArrayList<>();
            }
            return instanceList.stream()
                    .map(instance -> {
                        return InstanceInfo.builder()
                                .instanceId(instance.getInstanceId())
                                .instanceName(instance.getInstanceName())
                                .status(instance.getStatus())
                                .publicIpAddress(instance.getPublicIpAddress().getIpAddress().toString())
                                .privateIpAddress(instance.getVpcAttributes().getPrivateIpAddress().ipAddress.toString())
                                .recyclable(instance.getRecyclable())
                                .creationTime(instance.creationTime)
                                .build();

                    }).toList();
        } catch (TeaException error) {
            // 此处仅做打印展示，请谨慎对待异常处理，在工程项目中切勿直接忽略异常。
            // 错误 message
            log.error("状态码是:[{}],信息是:[{}],返回的结果是:[{}]", error.getCode(), error.getMessage(), error);
        } catch (Exception _error) {
            TeaException error = new TeaException();
            log.error("code:[{}],信息是:[{}],返回的结果是:[{}]",error.getCode(), error.getMessage(), error);
        }

        return new ArrayList<>();
    }

    //todo----------------------------------------------------以下是新增代码--------------------------------------------

    /**
     * 新增实列
     */
    /**
     * initialization 初始化公共请求参数
     */
    public List<String> startCreate() throws Exception {
        // 公网出带宽最大值，单位为 Mbit/s。取值范围：0~100。 默认值：0。
        Integer internetMaxBandwidthOut = com.aliyun.darabonbanumber.Client.parseInt("5");
        // 批量创建实例
        List<String> s = RunInstances(
                client, alyConfigProperties.getRegionId(), alyConfigProperties.getImageId(), alyConfigProperties.getInstanceType(),
                alyConfigProperties.getSecurityGroupId(), alyConfigProperties.getVSwitchId(), internetMaxBandwidthOut,
                alyConfigProperties.getInternetChargeType(), alyConfigProperties.getSize(), alyConfigProperties.getCategory(),
                alyConfigProperties.getInstanceChargeType());
        EcsSelectModel ecsSelectModel = new EcsSelectModel();
        ecsSelectModel.setInstanceIdList(s);
        List<InstanceInfo> list = selectList(ecsSelectModel);
        list.forEach(
                item -> {
                    String publicIP = item.getPublicIpAddress().substring(1, item.getPublicIpAddress().length() - 1);
                    item.setPublicIpAddress(publicIP);
                    //存入数据
                    gatewayNodeSetVinCache.newInstance(item);
                    log.info("公网IP:" + item.getPublicIpAddress());
                }
        );
        return s;
    }

    /**
     * RunInstances    通过备选实例规格创建ECS实例最佳实践
     * 该场景中，在调用RunInstances创建ECS实例时判断是否发生库存不足等错误，如果发生错误，将调用DescribeRecommendInstanceType查询备选实例，然后通过备选实例规格重新创建ECS实例。
     */
    public List<String> RunInstances(Client client, String regionId, String imageId, String instanceType, String securityGroupId, String vSwitchId, Integer internetMaxBandwidthOut,
                                     String internetChargeType, String size, String category, String instanceChargeType) throws Exception {
        RunInstancesResponse responces =null;
        int i = 0;
        RunInstancesRequest request1 = new RunInstancesRequest()
                .setRegionId(regionId)
                .setImageId(imageId)
                .setInstanceType(instanceType)
                .setSecurityGroupId(securityGroupId)
                .setVSwitchId(vSwitchId)
                .setInstanceName("MyFirstEcsInstance")
                .setDescription("MyFirstEcsInstance")
                .setInternetMaxBandwidthOut(internetMaxBandwidthOut)
                .setInternetChargeType(internetChargeType)
                .setInstanceChargeType(instanceChargeType)
                // 批量创建五台ECS实例，如果不设置该参数，默认创建一台ECS实例。
                // amount = 5,
                // 如果缺少库存可以接受的最低创建数量。
                // minAmount = 2,
                // 打开预检参数功能，不会实际创建ECS实例，只检查参数正确性、用户权限或者ECS库存等问题。
                // 实际情况下，设置了DryRun参数后，Amount必须为1，MinAmount必须为空，您可以根据实际需求修改代码。
                .setDryRun(false)
                .setSystemDisk(new RunInstancesRequest.RunInstancesRequestSystemDisk()
                        .setSize(size)
                        .setCategory(category));
        try {
            com.aliyun.teaconsole.Client.log("--------------------批量创建实例开始--------------------");
            responces = client.runInstances(request1);
            com.aliyun.teaconsole.Client.log("--------------------创建实例成功，实例ID:" + (responces.body.instanceIdSets.instanceIdSet) + "--------------------");
            i++;
        } catch (TeaException error) {
            com.aliyun.teaconsole.Client.log("--------------------创建实例失败:" + Common.toJSONString(error.message) + "--------------------");
        } catch (Exception _error) {
            TeaException error = new TeaException(_error.getMessage(), _error);
            com.aliyun.teaconsole.Client.log("--------------------创建实例失败:" + Common.toJSONString(error.message) + "--------------------");
        }
        return responces.body.instanceIdSets.instanceIdSet;
    }
    //todo----------------------------------------------------以下是删除代码----------------------------------------------
    /**
     * 删除
     */
    public DescribeInstancesResponse describeInstances(Client client,String regionId,String instanceIds,String instanceName) {
        DescribeInstancesRequest describeInstancesRequest = new DescribeInstancesRequest()
                .setRegionId(regionId)
                .setInstanceName(instanceName);
        if (!Common.empty(instanceIds)){
            describeInstancesRequest.instanceIds = Common.toJSONString(com.aliyun.darabonbastring.Client.split(instanceIds, ",", 50));
        }
        DescribeInstancesResponse resq = null;
        try {
            resq = client.describeInstances(describeInstancesRequest);
            com.aliyun.teaconsole.Client.log("--------------------查询需要删除的实例--------------------");
            return resq;
        } catch (Exception e) {
            throw new RuntimeException(e);
        }

    }

    public void modifyInstanceAttribute(Client client,String instatnceId) {
        ModifyInstanceAttributeRequest req = new ModifyInstanceAttributeRequest()
                .setInstanceId(instatnceId)
                .setDeletionProtection(false)
                ;
        try {
            client.modifyInstanceAttribute(req);
            com.aliyun.teaconsole.Client.log("--------------------"+instatnceId+"释放保护成功--------------------");
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    public void deleteInstances(Client client, String regionId, String instanceIds, String force)  {
        DeleteInstancesRequest req = new DeleteInstancesRequest()
                .setRegionId(regionId)
                .setInstanceId(com.aliyun.darabonbastring.Client.split(instanceIds, ",", 50))
                .setForce(Common.equalString(force, "true"));
        DeleteInstancesResponse resp = null;
        try {
            resp = client.deleteInstances(req);
            com.aliyun.teaconsole.Client.log("--------------------实例释放成功--------------------");
            com.aliyun.teaconsole.Client.log(Common.toJSONString(Common.toMap(resp)));
        } catch (Exception e) {
            throw new RuntimeException(e);
        }

    }

    public String delete (String instanceIds) {
        //实例名称
        String instanceName = "MyFirstEcsInstance";
        //强制删除有删除保护的机器
        String deleteProtected = "true";
        //强制删除运行中的机器
        String force = "true";

        if (Common.equalString(deleteProtected,"true")){
            DescribeInstancesResponse describeInstances = describeInstances(client, alyConfigProperties.getRegionId(), instanceIds, instanceName);
            instanceIds="";
            for (DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstancesInstance instance : describeInstances.body.instances.instance) {
                instanceIds = "" + instance.instanceId + "," + instanceIds + "";
                if (instance.deletionProtection) {
                    modifyInstanceAttribute(client, instance.instanceId);
                }
            }
            instanceIds = com.aliyun.darabonbastring.Client.subString(instanceIds, 0, -1);
        }
        if (Common.empty(instanceIds)){
            try {
                com.aliyun.teaconsole.Client.log("--------------------无有效实例可删除--------------------");
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
        deleteInstances(client,alyConfigProperties.getRegionId(),instanceIds,force);
        return instanceIds;
    }
}
