package com.yao.common.config;

import com.aliyun.ecs20140526.Client;
import com.aliyun.teaopenapi.models.Config;
import lombok.Data;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * @Author: LiJiaYao
 * @Date: 2024/4/16
 * @Description: 阿里云配置
 */
@Configuration
@ConfigurationProperties(prefix = "config.aly")
@Data
public class AlyConfigProperties {

    private String accessKeyId;
    private String accessKeySecret;
    /**
     * 地域id
     */
    private String regionId;
    private String imageId;

    /*
    实列规格
     */
    private String instanceType;
    /**
     * 安全组
     */
    private String securityGroupId;
    /**
     * 虚拟交换机
     */
    private String vSwitchId;

    /**
     * 网络计费类型
     */
    private String internetChargeType;
    /**
     * 系统盘大小
     */
    private String size;
    /**
     * 系统盘的云盘种类
     */
    private String category;
    //ECS实例的计费方式
    private String instanceChargeType;

    /**
     * @param alyConfig
     * @return
     * @throws Exception
     * @Description: 创建一个单例
     */
    @Bean
    public Client createEcsClient(AlyConfigProperties alyConfig) throws Exception {
        Config config = new Config()
                // 您的AccessKey ID
                .setAccessKeyId(alyConfig.getAccessKeyId())
                // 您的AccessKey Secret
                .setAccessKeySecret(alyConfig.getAccessKeySecret())
                // 您的可用区ID
                .setRegionId(alyConfig.getRegionId());
        return new Client(config);
    }






}
