package com.yao.gateway.cache;

import com.yao.gateway.cache.abs.GatewayNodeAbstract;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @Author: LiJiaYao
 * @Date: 2024/4/18
 * @Description: 网关负载节点缓存
 */
@Component
public class GatewayLoadNodeCache extends GatewayNodeAbstract {


    private final static  String gateWayLoadNodeKey="gateway:load:node";

    /**
     * 存负载集合
     * @param nodeList 节点权重集合
     */
    public void put(List<String> nodeList){
        redisService.deleteObject(gateWayLoadNodeKey);
        redisService.setCacheList(gateWayLoadNodeKey,nodeList);
    }

    /**
     * 获取所有负载节点
     * @return 所有负载节点集合
     */
    public List<String> get(){
        return redisService.getCacheList(gateWayLoadNodeKey);
    }

    /**
     * 根据下标查询内容
     * @param index
     * @return
     */
    public String getByIndex(Long index){
        if (index == null || index > 100){
            throw new RuntimeException("下标违法,0-100");
        }
        return redisService.getCacheListValue(gateWayLoadNodeKey,index);
    }

}
