package com.yao.gateway.cache;

import com.yao.common.domain.GateWayNodeInfo;
import com.yao.common.domain.WorkGateWayNode;
import com.yao.common.domain.aliy.InstanceInfo;
import com.yao.gateway.cache.abs.GatewayNodeAbstract;
import org.springframework.data.redis.core.ZSetOperations;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @Author: LiJiaYao
 * @Date: 2024/4/18
 * @Description: 网关节点分数
 */
@Component
public class GatewayNodeScoreCache extends GatewayNodeAbstract {

    private static final String zSetKey = "gateway:zSet:count";

    /**
     * 网关连接计数
     *
     * @param info  新增数据的信息
     * @param count 网关连接的个数
     */
    public void newCount(InstanceInfo info, Integer count) {
        redisService.setCacheZSet(zSetKey, info, count);
    }

    /**
     * 取出数据
     *
     * @return 取出的内容
     */
    public Set<InstanceInfo> get() {
        Set<InstanceInfo> cacheZSet = redisService.getCacheZSet(zSetKey);

        return cacheZSet;
    }

    /**
     * 获取他的内容
     * @return
     */
    public List<GateWayNodeInfo> getCacheZSet() {
        Set<ZSetOperations.TypedTuple<String>> set = redisService.getZSet(zSetKey);
//        Map<String, Double> nodeMap = set.stream()
//                .collect(Collectors.toMap(
//                        ZSetOperations.TypedTuple::getValue,
//                        ZSetOperations.TypedTuple::getScore
//                ));
        return set.stream()
                .map(zset->
                    GateWayNodeInfo.builder()
                            .nodeId(zset.getValue())
                            .score(zset.getScore())
                            .build()
                ).toList();
        //算法：有没有超过百分比
//        double sum = nodeMap.values().stream().mapToDouble(value -> value).sum();
    }

    /**
     * 删除数据
     */
    public void remove() {
        redisService.deleteObject(zSetKey);
    }

}
