package com.yao.server.service.impl;

import com.yao.common.config.Constants;
import com.yao.common.domain.WorkGateWayNode;
import com.yao.common.mqtt.MqttConnectService;
import com.yao.gateway.cache.GatewayArithmeticCache;
import com.yao.gateway.cache.GatewayVehicleLineNodeCache;
import com.yao.server.service.LoadService;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.CountDownLatch;

/**
 * @Author: LiJiaYao
 * @Date: 2024/4/14
 * @Description:
 */

@Log4j2
@Service
public class LoadServiceImpl implements LoadService {
    //网关算法缓存
    private final GatewayArithmeticCache gatewayArithmeticCache;
    // 网关连接车俩
    private final GatewayVehicleLineNodeCache gatewayVehicleLineNodeCache;
    private MqttConnectService mqttConnectService;

    public LoadServiceImpl(GatewayArithmeticCache gatewayArithmeticCache, GatewayVehicleLineNodeCache gatewayVehicleLineNodeCache, MqttConnectService mqttConnectService) {
        this.gatewayArithmeticCache = gatewayArithmeticCache;
        this.gatewayVehicleLineNodeCache = gatewayVehicleLineNodeCache;
        this.mqttConnectService = mqttConnectService;
    }

    @Override
    public String load() {
        //初始化序列
        gatewayArithmeticCache.count();
        ArrayList<WorkGateWayNode> nodeIdList = carWorkGatewayNode();
        //100
        List<String> loadNodeList = new ArrayList<>();
        Integer count = nodeIdList.stream().mapToInt(WorkGateWayNode::getWeight).sum();
        if (count < Constants.SUM) {
            List<WorkGateWayNode> list = nodeIdList.stream()
                    .sorted(((o1, o2) -> o2.getWeight() - o1.getWeight()))
                    .toList();
            Integer countWeight = 0;
            for (Integer i = count; i < Constants.SUM; i++) {
                WorkGateWayNode workGateWayNode = list.get(countWeight++ % list.size());
                workGateWayNode.setWeight(workGateWayNode.getWeight() + 1);
            }
        }

        work:
        while (true) {
            for (WorkGateWayNode workGateWayNode : nodeIdList) {
                Integer nodeWeight = workGateWayNode.getWeight();
                if (nodeWeight > 0) {
                    loadNodeList.add(
                            workGateWayNode.getNodeId()
                    );
                    workGateWayNode.setWeight(nodeWeight - 1);
                }
                int intStream = nodeIdList.stream()
                        .mapToInt(WorkGateWayNode::getWeight)
                        .sum();
                if (intStream <= 0) {
                    break work;
                }
            }
        }
        gatewayArithmeticCache.remove();
        gatewayArithmeticCache.loadNode(loadNodeList);
        CountDownLatch countDownLatch = new CountDownLatch(300);
        new Thread(() -> {
            for (int i = 0; i < Constants.SUM; i++) {
                long cursor = gatewayArithmeticCache.increment();
                String cacheList = gatewayArithmeticCache.cacheList(cursor);
                log.info(cursor + "---------" + cacheList);
                SitNode.sti(cacheList);
                countDownLatch.countDown();
            }
        }).start();

        new Thread(() -> {
            for (int i = 0; i < Constants.SUM; i++) {
                long cursor = gatewayArithmeticCache.increment();
                String cacheList = gatewayArithmeticCache.cacheList(cursor);
                log.info(cursor + "---------" + cacheList);
                SitNode.sti(cacheList);
                countDownLatch.countDown();
            }
        }).start();

        new Thread(() -> {
            for (int i = 0; i < Constants.SUM; i++) {
                long cursor = gatewayArithmeticCache.increment();
                String cacheList = gatewayArithmeticCache.cacheList(cursor);
                log.info(cursor + "---------" + cacheList);
                SitNode.sti(cacheList);
                countDownLatch.countDown();
            }
        }).start();
        try {
            countDownLatch.await();
            SitNode.show();
        } catch (
                InterruptedException e) {
            throw new RuntimeException(e);
        }

        HashMap<String, Integer> show = SitNode.show();
        // 找到最小值
        Map.Entry<String, Integer> minEntry = findMinValue(show);
        String key = null;
        if (minEntry != null) {
            log.info("最小值的键：" + minEntry.getKey());
            key = minEntry.getKey();
            log.info("最小值：" + minEntry.getValue());
        } else {
            System.out.println("HashMap 为空！");
        }
        return key;
    }

    public ArrayList<WorkGateWayNode> carWorkGatewayNode() {
        Set<String> ip = gatewayVehicleLineNodeCache.getAddress();
        ArrayList<WorkGateWayNode> list = new ArrayList<>();
        for (String s : ip) {
            Integer connectSize = mqttConnectService.connectMqtt(s);
            WorkGateWayNode workGateWayNode = new WorkGateWayNode();
            workGateWayNode.setWeight(connectSize);
            workGateWayNode.setNodeId(s);
            list.add(workGateWayNode);
        }
        return list;

    }

    //获取最小对应的key和value
    public static Map.Entry<String, Integer> findMinValue(HashMap<String, Integer> map) {
        Map.Entry<String, Integer> minEntry = null;
        for (Map.Entry<String, Integer> entry : map.entrySet()) {
            if (minEntry == null || entry.getValue() < minEntry.getValue()) {
                minEntry = entry;
            }
        }
        return minEntry;
    }

}


//计数
@Log4j2
class SitNode {

    private static Map<String, Integer> sitNodeMap = new HashMap<>();

    public synchronized static void sti(String nodeId) {
        Integer orDefault = sitNodeMap.getOrDefault(nodeId, 0);
        sitNodeMap.put(nodeId, orDefault + 1);
    }

    public static HashMap<String, Integer> show() {
        HashMap<String, Integer> map = new HashMap<>();
        sitNodeMap.forEach((key, val) -> {
            map.put(key, val);
            log.info(key + "-------" + val);
        });

        return map;
    }
}
