package com.yao.server.timer;

import com.yao.common.aliy.AliYunEcsService;
import com.yao.common.domain.GateWayNodeInfo;
import com.yao.common.domain.aliy.InstanceInfo;
import com.yao.common.mqtt.MqttConnectService;
import com.yao.gateway.cache.GatewayNodeScoreCache;
import com.yao.gateway.cache.GatewayNodeSetVinCache;
import com.yao.gateway.cache.GatewayVehicleLineNodeCache;
import lombok.extern.log4j.Log4j2;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Set;

/**
 * @Author: LiJiaYao
 * @Date: 2024/4/13
 * @Description:
 */
@Log4j2
@EnableAsync //开启异步支持
@Component
public class Timer {
    private final AliYunEcsService aliYunEcsService;
    private final MqttConnectService mqttConnectService;
    // 阿里云缓存
    private final GatewayNodeSetVinCache gatewayNodeSetVinCache;
    private final GatewayVehicleLineNodeCache gatewayVehicleLineNodeCache;
    private final GatewayNodeScoreCache gatewayNodeScoreCache;

    public Timer(AliYunEcsService aliYunEcsService, MqttConnectService mqttConnectService, GatewayNodeSetVinCache gatewayAliYunCache, GatewayVehicleLineNodeCache gatewayVehicleLineNodeCache, GatewayNodeScoreCache gatewayNodeScoreCache) {
        this.aliYunEcsService = aliYunEcsService;
        this.mqttConnectService = mqttConnectService;
        this.gatewayNodeSetVinCache = gatewayAliYunCache;
        this.gatewayVehicleLineNodeCache = gatewayVehicleLineNodeCache;
        this.gatewayNodeScoreCache = gatewayNodeScoreCache;
    }

    @Async
    @Scheduled(cron = "0/20 * * * * ?")
    public void timer() {
        // 记录开始时间
        long startTime = System.currentTimeMillis();
        // 设置结束时间为10分钟后
        long endTime = startTime + 10 * 60 * 1000;
        Set<InstanceInfo> instance = gatewayNodeSetVinCache.get();
        if (instance.isEmpty()) {
            try {
                aliYunEcsService.startCreate();
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
        for (InstanceInfo s : instance) {
            Integer connectSize = mqttConnectService.connectMqtt(s.getPublicIpAddress());
            List<String> instanceId = null;
            String deleteInstanceId = null;
            if (connectSize >= 79) {
                //执行节点扩容
                //返回实例的ID
                if (!instanceId.isEmpty()) {
                    try {
                        instanceId = aliYunEcsService.startCreate();
                        log.info("扩容成功！");
                        log.info("扩容的节点id为：" + instanceId);
                    } catch (Exception e) {
                        throw new RuntimeException(e);
                    }
                }
            }
            if (connectSize <= 20 && System.currentTimeMillis() < endTime) {
                aliYunEcsService.delete(s.getInstanceId());
                //删除实列以后再去把redis的值删除 再去通知重新上线
                gatewayNodeSetVinCache.remote(s);
                gatewayVehicleLineNodeCache.save(s.getPublicIpAddress());
                log.info("缩容成功！");
                log.info("锁容的节点id为：" + deleteInstanceId);
            }
        }
    }
}
